/*
 * Copyright (c) 2005 The University of Notre Dame. All Rights Reserved.
 *
 * The University of Notre Dame grants you ("Licensee") a
 * non-exclusive, royalty free, license to use, modify and
 * redistribute this software in source and binary code form, provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. All express or implied conditions, representations and
 * warranties, including any implied warranty of merchantability,
 * fitness for a particular purpose or non-infringement, are hereby
 * excluded.  The University of Notre Dame and its licensors shall not
 * be liable for any damages suffered by licensee as a result of
 * using, modifying or distributing the software or its
 * derivatives. In no event will the University of Notre Dame or its
 * licensors be liable for any lost revenue, profit or data, or for
 * direct, indirect, special, consequential, incidental or punitive
 * damages, however caused and regardless of the theory of liability,
 * arising out of the use of or inability to use software, even if the
 * University of Notre Dame has been advised of the possibility of
 * such damages.
 *
 * SUPPORT OPEN SCIENCE!  If you use OpenMD or its source code in your
 * research, please cite the appropriate papers when you publish your
 * work.  Good starting points are:
 *                                                                      
 * [1]  Meineke, et al., J. Comp. Chem. 26, 252-271 (2005).             
 * [2]  Fennell & Gezelter, J. Chem. Phys. 124, 234104 (2006).          
 * [3]  Sun, Lin & Gezelter, J. Chem. Phys. 128, 234107 (2008).          
 * [4]  Kuang & Gezelter,  J. Chem. Phys. 133, 164101 (2010).
 * [5]  Vardeman, Stocker & Gezelter, J. Chem. Theory Comput. 7, 834 (2011).
 */
#ifndef APPLICATIONS_STATICPROPS_GOFRANGLE2_HPP
#define APPLICATIONS_STATICPROPS_GOFRANGLE2_HPP

#include "applications/staticProps/RadialDistrFunc.hpp"
namespace OpenMD {

  class GofRAngle2 : public RadialDistrFunc {
    
  public:
    GofRAngle2(SimInfo* info, const std::string& filename, 
               const std::string& sele1, const std::string& sele2, 
               RealType len, int nrbins, int nangleBins);
    GofRAngle2(SimInfo* info, const std::string& filename, 
               const std::string& sele1, const std::string& sele2, 
               const std::string& sele3,
               RealType len, int nrbins, int nangleBins);           
  private:

    virtual void preProcess();
    virtual void processNonOverlapping( SelectionManager& sman1,
                                        SelectionManager& sman2);
    virtual void processOverlapping( SelectionManager& sman );
    virtual void initializeHistogram();
    virtual void collectHistogram(StuntDouble* sd1, StuntDouble* sd2);
    virtual void collectHistogram(StuntDouble* sd1, StuntDouble* sd2, 
                                  StuntDouble* sd3);
    virtual void processHistogram();
    virtual void writeRdf();

    unsigned int nAngleBins_;
    RealType len_;
    RealType deltaR_;
    
    RealType deltaCosAngle_;

    bool doSele3_;
    SelectionManager seleMan3_;
    SelectionEvaluator evaluator3_;
    std::string selectionScript3_;
        
    std::vector<std::vector<std::vector<int> > > histogram_;
    std::vector<std::vector<std::vector<RealType> > > avgGofr_;
  };

}
#endif
