/*
 * Copyright (c) 2005 The University of Notre Dame. All Rights Reserved.
 *
 * The University of Notre Dame grants you ("Licensee") a
 * non-exclusive, royalty free, license to use, modify and
 * redistribute this software in source and binary code form, provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. All express or implied conditions, representations and
 * warranties, including any implied warranty of merchantability,
 * fitness for a particular purpose or non-infringement, are hereby
 * excluded.  The University of Notre Dame and its licensors shall not
 * be liable for any damages suffered by licensee as a result of
 * using, modifying or distributing the software or its
 * derivatives. In no event will the University of Notre Dame or its
 * licensors be liable for any lost revenue, profit or data, or for
 * direct, indirect, special, consequential, incidental or punitive
 * damages, however caused and regardless of the theory of liability,
 * arising out of the use of or inability to use software, even if the
 * University of Notre Dame has been advised of the possibility of
 * such damages.
 *
 * SUPPORT OPEN SCIENCE!  If you use OpenMD or its source code in your
 * research, please cite the appropriate papers when you publish your
 * work.  Good starting points are:
 *                                                                      
 * [1]  Meineke, et al., J. Comp. Chem. 26, 252-271 (2005).             
 * [2]  Fennell & Gezelter, J. Chem. Phys. 124, 234104 (2006).          
 * [3]  Sun, Lin & Gezelter, J. Chem. Phys. 128, 234107 (2008).          
 * [4]  Kuang & Gezelter,  J. Chem. Phys. 133, 164101 (2010).
 * [5]  Vardeman, Stocker & Gezelter, J. Chem. Theory Comput. 7, 834 (2011).
 */
 
#include "lattice/LatticeFactory.hpp"
#include "lattice/LatticeCreator.hpp"

namespace OpenMD {

  //initialize instance of LatticeFactory
  LatticeFactory* LatticeFactory::instance_ = NULL;

  LatticeFactory::~LatticeFactory() {
    CreatorMapType::iterator i;
    for (i = creatorMap_.begin(); i != creatorMap_.end(); ++i) {
      delete i->second;
    }
    creatorMap_.clear();
  }

  bool LatticeFactory::registerLattice(LatticeCreator* creator) {
    return creatorMap_.insert(
			      CreatorMapType::value_type(creator->getIdent(), creator)).second;
  }

  bool LatticeFactory::unregisterLattice(const std::string& id) {
    return creatorMap_.erase(id) == 1;
  }

  Lattice* LatticeFactory::createLattice(const std::string& id) {
    CreatorMapType::iterator i = creatorMap_.find(id);
    if (i != creatorMap_.end()) {
      //invoke functor to create object
      return (i->second)->create();
    } else {
      return NULL;
    }
  }

  std::vector<std::string> LatticeFactory::getIdents() {
    IdentVectorType idents;
    CreatorMapType::iterator i;

    for (i = creatorMap_.begin(); i != creatorMap_.end(); ++i) {
      idents.push_back(i->first);
    }
    
    return idents;
  }

  std::ostream& operator <<(std::ostream& o, LatticeFactory& factory) {
    LatticeFactory::IdentVectorType idents;
    LatticeFactory::IdentVectorIterator i;

    idents = factory.getIdents();

    o << "Avaliable type identifiers in this factory: " << std::endl;
    for (i = idents.begin(); i != idents.end(); ++i) {
      o << *i << std::endl;
    }

    return o;
  }

}
